import re
import random
import time
import requests
import datetime
# import pymysql

from typing import List
from faker import Faker
from commen.unit_logger import logger


class FunBase:
    def __init__(self):
        self._time2 = 0
        self._time1 = 0
        self.sql_str = ""
        # self.locale_list = {"zh_CN": "简体中文", "de_DE": "德语-德国", "sv_SE": "Swedish", "ja_JP": "日语-日本", "ko_KR": "朝鲜语-韩国"}
        # faker方法： http://www.manongjc.com/detail/28-mhkegnrmyzaafky.html
        self._faker = Faker(locale='zh_CN')
        self._db = {}
        self._route = ""
        self._token = ""
        self._host = ""

    # mock 方法，默认中国，详情搜python Faker模块
    def faker(self, locale='zh_CN'):
        return Faker(locale=locale)
    
    def seelp(self, v=1, *args, **kwargs):
        if isinstance(v, int):
            self.log_info("seelp: %s" % v)
            time.sleep(v)
        else:
            time.sleep(1)
            

    def _faker_name(self):
        return self._faker.name() + str(self._faker.numerify())

    @staticmethod
    def random_vin():
        # 内容的权值
        content_map = {
            'A': 1, 'B': 2, 'C': 3, 'D': 4, 'E': 5,
            'F': 6, 'G': 7, 'H': 8, 'I': 0, 'J': 1, 'K': 2, 'L': 3,
            'M': 4, 'N': 5, 'O': 0, 'P': 7, 'Q': 8, 'R': 9, 'S': 2, 'T': 3,
            'U': 4, 'V': 5, 'W': 6, 'X': 7, 'Y': 8, 'Z': 9, "0": 0, "1": 1,
            "2": 2, "3": 3, "4": 4, "5": 5, "6": 6, "7": 7, "8": 8, "9": 9
        }
        # 位置的全值
        location_map = [8, 7, 6, 5, 4, 3, 2, 10, 0, 9, 8, 7, 6, 5, 4, 3, 2]
        vin = ''.join(random.sample('0123456789ABCDEFGHJKLMPRSTUVWXYZ', 17))
        num = 0
        for i in range(len(vin)):
            num = num + content_map[vin[i]] * location_map[i]
        vin9 = num % 11
        if vin9 == 10:
            vin9 = "X"
        list1 = list(vin)
        list1[8] = str(vin9)
        vin = ''.join(list1)
        return vin

    @staticmethod
    def random_carnum():
        char0 = '京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽赣粤青藏川宁琼'
        char1 = 'ABCDEFGHJKLMNPQRSTUVWXYZ'  # 车牌号中没有I和O，可自行百度
        char2 = '1234567890ABCDEFGHJKLMNPQRSTUVWXYZ'
        char3 = '1234567890'
        while True:
            code = random.choice(char0)
            code += random.choice(char1)
            code += "".join(random.choices(char2, k=4))
            code += random.choice(char3)
            return code
            test = re.match('^.\w.[A-Z]\d{4}$|^.\w.\d[A-Z]\d{3}$|^.\w.\d{2}[A-Z]\d{2}$|^.\w.\d{3}[A-Z]\d$|^.\w.\d{5}$', code)
            if test:
                return code

    # # mock方法，国家locale_list随机，也可根据数字获取第几个国家，超出时随机
    # def faker_random_country(self, locale=None):
    #     country = None
    #     if locale:
    #         try:
    #             if self.locale_list.get(locale):
    #                 country = locale
    #             else:
    #                 country = list(self.locale_list.keys())[int(locale)]
    #         except:
    #             pass
    #     if not country:
    #         country = random.choice(list(self.locale_list.keys()))
    #     return self.faker(locale=country)

    @staticmethod
    def log_info(msg):
        logger.info(str(msg))

    def sleep(self, t=1, debug=False):
        self.log_info("sleep %s 秒" % t)
        n = 1
        while debug and t > 1:
            self.log_info("sleep 第 %s 秒" % n)
            time.sleep(1)
            n += 1
            t -= 1
        time.sleep(t)

    # 时间戳，秒级
    @staticmethod
    def time(string=None, fmt="%Y-%m-%d"):
        time_array = int(time.mktime(time.strptime(string, fmt))) if string else int(time.time())
        return time_array

    def time_hms(self, string=None, fmt="%Y-%m-%d %H:%M:%S"):
        return self.time(string=string, fmt=fmt) * 1000

    def time_ms(self, string=None, fmt="%Y-%m-%d"):
        time_array = self.time(string=string, fmt=fmt)
        return time_array * 1000

    def data_str(self):
        return datetime.datetime.now().strftime("%Y-%m-%d")
    # # 根据sql语句执行sql
    # def run_mysql(self, sql_str="", host="", user="", password="", database=""):
    #     sql_str = sql_str or self.sql_str
    #     if self._db.get(host+database):
    #         cursor = self._db.get(host+database)
    #     else:
    #         try:
    #             if database:
    #                 db = pymysql.connect(host=host, user=user, passwd=password, database=database)
    #             else:
    #                 db = pymysql.connect(host=host, user=user, passwd=password)
    #         except:
    #             assert False, "Couldn't connect to database"
    #         cursor = db.cursor()
    #         self._db[host+user+password+database] = cursor
    #     try:
    #         cursor.execute(sql_str)
    #
    #     except Exception as e:
    #         assert False, "执行sql失败，sql：" + sql_str + "\n执行报错：" + str(e)
    #
    #     return []

    # 传入表名加字段条件自动生成sql语句
    def get_sql_str(self, table: str, **kwargs):
        kwargs.update(kwargs.get('where') or {})
        where_str = "select * from " + table if len(table.split()) == 1 else table
        not_where_str = ('page', 'page_index', 'page_no', 'page_size', 'order_by', "where", "group_by", "having",
                         "where_list", "page_num", 'headers', '_route', 'current', 'size', '_token')
        where_if = [i for i in kwargs.keys() if i not in not_where_str and (kwargs.get(i) is not None or "null__" in i)]

        tmp_list = []
        for i in where_if:
            if "like__" in i:
                tmp_list.append("%s like \"%%%s%%\" " % (i[6:], kwargs.get(i)))
            elif "not_null__" in i:
                tmp_list.append("%s is not null " % (i[10:],))
            elif "null__" in i:
                tmp_list.append("%s is null " % (i[6:],))
            elif "in__" in i:
                if len(kwargs.get(i)) > 1:
                    tmp_list.append("%s in %s " % (i[4:], str(tuple(kwargs.get(i)))))
                elif len(kwargs.get(i)) == 1:
                    tmp_list.append("%s = \"%s\" " % (i[4:], str(kwargs.get(i)[0])))
                else:
                    tmp_list.append("%s = \"&$#@\" " % (i[4:], ))
            else:
                tmp_list.append("%s = \"%s\" " % (i, kwargs.get(i)))
        tmp_list.extend(kwargs.get("where_list") or [])
        if tmp_list:
            where_str += " where " + " and ".join(tmp_list)
        if kwargs.get("group_by"):
            where_str += " group by " + kwargs.get('group_by') + " "
        if kwargs.get("having"):
            where_str += " having " + kwargs.get('having') + " "
        if kwargs.get('order_by'):
            where_str += " order by " + kwargs.get('order_by') + " "
        if where_str.strip()[:6].lower() == "select":
            page = kwargs.get("page") or kwargs.get("page_no") or kwargs.get("page_index") or \
                   kwargs.get("page_num") or kwargs.get('current') or 1
            page_size = kwargs.get("page_size") or kwargs.get('size') or 100
            where_str += " limit %s, %s;" % ((page - 1) * page_size, page_size)
        else:
            where_str += ";"
        self.sql_str = where_str
        return where_str

    # 传入2个json，比较值是否一致，compare_list格式: [("res1_key1", "res2_key1"), ...]
    def compare_json(self, res1: dict, res2: dict, key_list=[]):
        if isinstance(res1, requests.Response):
            res1 = res1.rsp
            if "data" in res1.keys():
                res1 = res1["data"]
        self.log_info("比较：res1=" + str(res1))
        self.log_info("     res2=" + str(res2))
        key_list = key_list or res1.keys()
        self.log_info("校验字段：" + str(key_list))
        for key in key_list:
            try:
                if type(key) == str:
                    key1 = key
                    if key in res2.keys():
                        key2 = key
                    else:
                        key2 = re.sub(r'[A-Z]', lambda n: "_" + str(n.group()).lower(), key)
                else:
                    key1 = key[0]
                    key2 = key[1]
                if not(not res1[key1] and not res2[key2]):
                    if type(res1[key1]) == int:
                        res2[key2] = int(res2[key2])
                    elif type(res1[key1]) == float:
                        res2[key2] = float(res2[key2])
                    if not(res1[key1] == res2[key2] or str(res1[key1]).strip() == str(res2[key2]).strip() or
                           (not res1[key1] and not res2[key2])):
                        if not ("time" in key1.lower() and len(str(res1[key1])) == 19 and len(str(res2[key2])) == 19
                                and str(res1[key1])[0:10] == str(res2[key2])[0:10]
                                and str(res1[key1])[11:] == str(res2[key2])[11:]):
                            self.log_info("比较字段：" + str(key) + "  不一致")
                            return False
            except Exception as e:
                self.log_info("比较字段：" + str(key) + "  不一致")
                return False
        self.log_info("比较结果一致")
        return True

    # 传入2个json列表，比较值是否一致，compare_list格式: [("res1_key1", "res2_key1"), ...]
    def compare_json_list(self, res1, res2: List[dict], key_list=[]):
        if isinstance(res1, requests.Response):
            res1 = res1.rsp.data
            if isinstance(res1, dict):
                for key in res1.keys():
                    if isinstance(res1[key], list):
                        res1 = res1[key]
                        break
        self.log_info("比较数组：res1=" + str(res1))
        self.log_info("        res2=" + str(res2))
        if not res1 and not res2:
            return True
        elif len(res1) == len(res2):
            if len(res1) == 0:
                return True
            flag = True
            for v1, v2 in zip(res1, res2):
                flag = self.compare_json(v1, v2, key_list)
                if not flag:
                    break
            return flag
        else:
            self.log_info("比较结果：比较数组个数不一致")
            try:
                key = key_list[0] if key_list else res1[0].keys()[0]
                if type(key) == str:
                    key1 = key
                    if key in res2[0].keys():
                        key2 = key
                    else:
                        key2 = re.sub(r'[A-Z]', lambda n: "_" + str(n.group()).lower(), key)
                else:
                    key1 = key[0]
                    key2 = key[1]
                id_list1 = [str(i[key1]) for i in res1]
                id_list2 = [str(i[key2]) for i in res2]
                self.log_info("    res1[%s]: %s" % (key1, id_list1))
                self.log_info("    res2[%s]: %s" % (key2, id_list2))
                diff1 = list(set(id_list1).difference(set(id_list2)))
                diff2 = list(set(id_list2).difference(set(id_list1)))
                if diff1:
                    self.log_info("    res1中有res2中没有的%s: %s" % (key1, diff1))
                if diff2:
                    self.log_info("    res2中有res1中没有的%s: %s" % (key2, diff2))
            except:
                pass
            return False

    @staticmethod
    def _random_choice(list_or_dict, _type=1):
        if list_or_dict:
            if isinstance(list_or_dict, (list, tuple, type({}.keys()))):
                out = random.choice(list(list_or_dict))
            elif isinstance(list_or_dict, dict):
                out = random.choice(list(list_or_dict.keys()))
            else:
                return list_or_dict
            if _type == 2 and isinstance(out, str) and len(out) > 2:
                i = random.randint(0, 2)
                if i == 0:
                    return out[1:]
                elif i == 1:
                    return out[:-1]
                else:
                    return out[1:-1]
            return out
        else:
            return None

    # 列表或字典里随机取值, 列表接口取值随机取值用
    def get_list_choice(self, *args, list_or_dict=None, _type=1):
        if list_or_dict is None and isinstance(args[-1], (list, tuple, type({}.keys()))):
            list_or_dict = args[-1]
            args = args[:-1]

        out = None
        for i in args[1:]:
            out = out or i
        if args[0] == '$None$':
            return None or out
        elif args[0] is True:
            if list_or_dict:
                return self._random_choice(list_or_dict, _type=_type)
            else:
                return out
        else:
            return args[0]

    # 字典里取随机字段，截取一段用于随机查询
    def get_list_choice2(self, *args, list_or_dict=None):
        return self.get_list_choice(*args, list_or_dict=None, _type=2)

    def get_value_choice(self, *args, list_or_dict=None, _all_is_None=False):
        if list_or_dict is None and isinstance(args[-1], (list, tuple, type({}.keys()))):
            list_or_dict = args[-1]
            args = args[:-1]
        if args[0] != '$None$':
            return args[0]
        else:
            if _all_is_None:
                return None
            out = None
            for i in args[1:]:
                out = out or i
            if out is None:
                if list_or_dict:
                    return self._random_choice(list_or_dict)
                else:
                    return None
            else:
                return out

    # 随机获取两位小数, 或空
    def get_random_float(self, null=False):
        if null:
            return random.randint(1, 99999) / 100 if self._faker.boolean() else None
        else:
            return random.randint(1, 99999) / 100

    # 时间获取时间
    def get_random_date(self, type=0, null=False):
        """type：0 当天，1：后一个月内, 2:后一年内, 3:继上次一个月后， 4：继上次一年后，
                 5：上两次之间, """

        if type == 0:
            self._time2 = self._time1
            self._time1 = 0
        elif type == 1:
            self._time2 = self._time1
            self._time1 = random.randint(0, 30)
        elif type == 2:
            self._time2 = self._time1
            self._time1 = random.randint(0, 365)
        elif type == 3:
            self._time2 = self._time1
            self._time1 += random.randint(0, 30)
        elif type == 4:
            self._time2 = self._time1
            self._time1 += random.randint(0, 365)
        elif type == 5:
            tmp = random.randint(self._time2, self._time1)\
                if self._time1 > self._time2 else random.randint(self._time1, self._time2)

            self._time2 = self._time1
            self._time1 = tmp

        data = str(self._faker.date_between(self._time1, self._time1))
        return (data if self._faker.boolean() else None) if null else data

    @staticmethod
    def kwargs_replace(kwargs, likes=[], ins=[], joins=[], before_end=[]):
        for i in likes:
            try:
                kwargs["like__" + i] = kwargs.pop(i)
            except:
                pass
        for i in ins:
            try:
                kwargs["in__" + i] = kwargs.pop(i)
            except:
                pass
        for i in joins:
            try:
                kwargs[i] = kwargs.pop(i.split('.')[1])
            except:
                pass
        for i in before_end:
            try:
                kwargs["where_list"] = kwargs.get("where_list", [])
                if "before" == i.lower()[-6:] and kwargs.get(i) is not None:
                    kwargs["where_list"].append(' %s >= "%s"' % (i[:-6], kwargs.pop(i)))
                elif "end" == i.lower()[-3:] and kwargs.get(i) is not None:
                    kwargs["where_list"].append(' %s <= "%s"' % (i[:-3], kwargs.pop(i)))
            except:
                pass
        return kwargs

    @staticmethod
    def kwargs_pop(kwargs, name):
        try:
            return kwargs.pop(name)
        except:
            return None

    def get_kwargs(self, kwargs):
        kwargs.update(kwargs.get('kwargs', {}))
        _kwargs = {k: v for k, v in kwargs.items() if (k not in ('self', 'kwargs') and k[0] != '_')
                 or k in ('_route', '_token', '_host')}
        if not kwargs.get('_route') and self._route:
            _kwargs['_route'] = self._route
        if not kwargs.get('_token') and self._token:
            _kwargs['_token'] = self._token
        if not kwargs.get('_host') and self._host:
            _kwargs['_host'] = self._host
        return _kwargs

    @staticmethod
    def has_true(kwargs):
        _kwargs = {k: v for k, v in kwargs.items() if k not in ('self', 'kwargs') and k[0] != '_' and v is True}
        return _kwargs

    def get_res_value_list(self, name, property_path=''):
        if property_path:
            tmp = self.res.rsp
            for i in property_path.split('.'):
                if not isinstance(tmp, dict):
                    return []
                tmp = tmp.get(i)
            if isinstance(tmp, list) and tmp and isinstance(tmp[0], dict):
                return [i.get(name) for i in tmp if i.get(name)]
            return []
        else:
            if self.res.status_code == 200 and self.res.rsp and isinstance(self.res.rsp, dict):
                for key, value in self.res.rsp.items():
                    if isinstance(value, list):
                        return [i.get(name) for i in value]
                    elif isinstance(value, dict):
                        for key2, value2 in value.items():
                            if isinstance(value2, list):
                                return [i.get(name) for i in value2 if i.get(name)]
            return []

    def excelWriter(self, filename, data):
        from pandas import DataFrame, ExcelWriter
        filename = filename[:4] + filename[4:].replace(":", "").replace(' ', '_')
        with ExcelWriter(filename) as writer:
            # 写入到第一个sheet
            if data and isinstance(data[0], list):
                h = data[0]
                data1 = []
                for i in data[1:]:
                    data1.append({x[0]: str(x[1]) for x in zip(h, i)})
                data = data1
            elif data and isinstance(data[0], dict):
                data = [{k: str(v) for k, v in i.items()} for i in data]
            DataFrame(data).to_excel(writer,
                              sheet_name="sheet1",
                              index=False,
                              engine="openpyxl"
                              )


if __name__ == '__main__':
    print(FunBase().random_carnum())
    # f = FunBase().faker()

    # print(f.name())  # 随机姓名
    # print(f.ipv4())  # 随机IP4地址
    # print(f.uri())  # 随机URI地址
    # print(f.url())  # 随机URL地址
    # print(f.user_name())  # 随机⽤户名
    # print(f.paragraph())  # 随机⽣成⼀个段落
    # print(f.text())  # 随机⽣成⼀篇⽂章
    # print(f.word())  # 随机⽣成词语
    # print(f.boolean())  # True/False
    # print(f.email())  # email
    # print(f.safe_email())  # 安全邮箱
    # print(f.ssn())  # ⽣成⾝份证号
    # print(f.password())  # 随机⽣成密码,可选参数：length：密码长度；special_chars：是否能使⽤特殊字符；digits：是否包含数字；upper_case：是否包含⼤写字母；lower_case：是否包含⼩写字母
    # print(f.numerify())  # ⽣成三位随机数
    # print(f.random_digit())  # ⽣成0~9随机数
    # print(f.random_int())  # 随机数字，默认0~9999，可通过min,max参数修改
    # print(f.random_letter())  # 随机字母
    # print(f.random_number(digits=12))  # 参数digits设置⽣成的数字位数
    # print(f.date())  # 随机⽇期
    # print(f.datez())  # 随机时间
    # print(f.date_between('-1M', 0))  # 随机⽣成指定范围内⽇期，参数：start_date，end_date
    # print(f.future_date())  # 未来⽇期
    # print(f.past_date())  # 随机⽣成已经过去的⽇期



import re
import requests
import json
import time
import urllib
import os
import hashlib

from requests import ReadTimeout, ConnectTimeout
from requests.exceptions import MissingSchema
from urllib.parse import urlencode
from commen.unit_dict import Dict
from commen.unit_logger import logger
from openpyxl.reader.excel import load_workbook
from commen import admin_host, projects_path, files_path


class UnitRequest:
    def __init__(self, ZEST_ENV=None, APP_KEY=None, TIMEOUT=30000):
        self.ZEST_ENV = ZEST_ENV
        self.APP_KEY = APP_KEY
        self.TIMEOUT = TIMEOUT
        self.variable = self.get_variable()
        
    
    @staticmethod
    def get_url(url, kwargs):
        tmp = re.findall(r'\{.+}', url)
        if not tmp:
            return url
        if kwargs.get(tmp[0][1:-1]):
            url = url.replace(tmp[0], kwargs.get(tmp[0][1:-1]))
            url = url.split('{')[0]
            return url
        else:
            assert False, "字段：" + tmp[0][1:-1] + "，为必填字段"
    
    @staticmethod
    def unit_http_requester(method, url, params=None, data=None, headers=None, host=None, timeout=10000,
                            init_headers={},
                            _route="", _files=None, **kwargs):
        if headers.get("headers"):
            _headers = init_headers
            tmp = headers.pop("headers")
            _headers.update(headers)
            if isinstance(tmp, dict):
                _headers.update(tmp)
        else:
            headers.pop("headers")
            _headers = headers

        if url[0] == '/' and host:
            if _route and _route[0] != '/':
                _route = '/' + _route
            url = host + _route + url
        if method.upper() == "GET":
            params = params or data
        logger.info("接口请求参数method: %s, url: %s" % (method, url))
        logger.info("接口请求参数headers: %s" % str(_headers))
        logger.info("接口请求参数params: %s" % str(params)) if params else logger.info(
            "接口请求参数data: %s" % str(data))
        if "//" not in url:
            assert False, "url没有域名"
        time_s = time.time()
        try:
            if "application/x-www-form-urlencoded" in str(_headers):
                data = urlencode(data)
                res = requests.request(method, url, data=data, headers=_headers, timeout=timeout / 1000,
                                       verify=False)
            elif "multipart/form-data" in str(_headers) and "file" in data.keys():
                file = str(data.pop("file"))
                if not os.path.isfile(file):
                    try:
                        out = requests.get(admin_host + '/files/file/', params={"value": file})
                        if "filename=" in str(out.raw.headers):
                            file_name = urllib.parse.unquote(re.findall(r'filename=(.+?)\'', str(out.raw.headers))[0])
                            file = os.path.join(files_path, file_name)
                            with open(file, 'wb') as f:
                                f.write(out.content)
                    except:
                        pass
                    if os.path.isfile(file):
                        pass
                    elif os.path.isfile(os.path.join(files_path, file)):
                        file = os.path.join(files_path, file)
                    elif os.path.isfile(os.path.join(projects_path, file)):
                        file = os.path.join(projects_path, file)
                    else:
                        assert False, "对应的文件不存在"
                data.update({'md5': hashlib.md5('%d_%d'.encode() % (0, int(time.time()))).hexdigest(),
                             'filesize': len(open(file, 'rb').read()), })
                _files = {"file": (os.path.basename(file), open(file, 'rb'))} if file else {}
                try:
                    del _headers['Content-Type']
                except:
                    pass
                if kwargs.get('cookies'):
                    res = requests.request(method, url, params=params, data=data, files=_files,
                                           headers=_headers, timeout=timeout / 1000, verify=False,
                                           cookies=kwargs.get('cookies'))
                else:
                    res = requests.request(method, url, params=params, data=data, files=_files,
                                           headers=_headers, timeout=timeout / 1000, verify=False)
            else:
                if kwargs.get('cookies'):
                    res = requests.request(method, url, params=params, json=data, headers=_headers,
                                           timeout=timeout / 1000,
                                           verify=False, cookies=kwargs.get('cookies'))
                else:
                    res = requests.request(method, url, params=params, json=data, headers=_headers,
                                           timeout=timeout / 1000,
                                           verify=False)

            logger.info("接口响应时间: %.3f毫秒" % (time.time() - time_s))
            try:
                res.rsp = Dict(json.loads(res.text)) if res.text else Dict(
                    {"code": -1, "msg": "该接口返回的数据为空字符串"})
            except:
                if "filename=" in str(res.raw.headers):
                    file_name = urllib.parse.unquote(re.findall(r'filename=(.+?)\'', str(res.raw.headers))[0])
                    if "xlsx" in file_name:
                        file_path = os.path.join(files_path, file_name)
                        with open(file_path, 'wb') as f:
                            f.write(res.content)
                        wb = load_workbook(file_path)
                        ws = wb.active
                        rows = [[cell.value for cell in row] for row in ws]
                        try:
                            data = [{c1: c2 for c1, c2 in zip(rows[0], row)} for row in rows[1:]]
                        except:
                            data = []
                        os.remove(file_path)
                        res.rsp = Dict({"code": 0, "data": data, "filename": file_name, "rows": rows})
                    else:
                        res.rsp = res.text
                else:
                    res.rsp = res.text
            logger.info("接口返回http_code: %s" % res.status_code)
            logger.info("接口数据res: %s" % str(res.rsp))
            return res
        except ReadTimeout:
            assert False, "接口响应时间%.3f秒, 超过设置的时间%.3f秒" % (time.time() - time_s, timeout / 1000)
        except ConnectTimeout:
            assert False, "接口响应时间%.3f秒, 超过设置的时间%.3f秒" % (time.time() - time_s, timeout / 1000)
        except MissingSchema:
            assert False, "url不对，无域名"
        except Exception as e:
            print(e)
            assert False, "调用接口报错"

    def get_variable(self, app_key=None, env=None):
        app_key = app_key or self.APP_KEY
        env = env or self.ZEST_ENV
        project = requests.post(admin_host + '/variable/variable/',
                                json={"app_key": app_key, "environment": env}).json()
        _variable = {}
        for v in project.get('variables', []):
            if v.get('type') == 0:
                _variable[v.get('name')] = v.get('value')
            elif v.get('type') == 1:
                _variable[v.get('name')] = json.loads(v.get('value'))
            elif v.get('type') == 2:
                _variable[v.get('name')] = int(json.loads(v.get('value')))
        _variable["_token"] = project.get("token")
        return _variable

    def init_headers(self, **kwargs):
        headers = {"Content-Type": "application/json"}
        _host = kwargs.get("_host", "HOST")
        host = self.variable.get(_host) if "//" not in _host else _host
        if host:
            headers.update({"referer": host})
        _token = kwargs.get("_token", "TOKEN")
        token = self.variable.get(_token) if len(_token) < 20 else _token
        if not token and self.variable.get("_token"):
            token = self.variable.get("_token").get(_token)
            if not token:
                token = self.variable.get("_token").get("token")
        if token:
            headers.update({"authorization": token})
        return headers
    
    def http_requester(self, *args, **kwargs):
        _host = kwargs.get("_host", "HOST")
        host = self.variable.get(_host) if "//" not in _host else _host
        return self.unit_http_requester(*args, host=host, timeout=self.TIMEOUT, init_headers=self.init_headers(**kwargs), **kwargs)


get_url = UnitRequest.get_url
unit_http_requester = UnitRequest.unit_http_requester




"""
    加解密
"""

import binascii
from pyDes import des, CBC, PAD_PKCS5
import base64
from Crypto.Cipher import PKCS1_v1_5
from Crypto import Random
from Crypto.PublicKey import RSA


def des_encryption(value, KEY=b'huhk0001'):
    """
    字符串，des加密
    :param value: str
    :param KEY: str 非必填
    :return: str
    """
    des_obj = des(KEY, CBC, KEY, pad=None, padmode=PAD_PKCS5)
    secret_bytes = des_obj.encrypt(value.encode(), padmode=PAD_PKCS5)
    return str(binascii.b2a_hex(secret_bytes), encoding="utf8")


def des_decryption(value, KEY=b'huhk0001'):
    """
    字符串，des解密
    :param value: str
    :param KEY: 非必填，需和加密key一致
    :return: str
    """
    des_obj = des(KEY, CBC, KEY, pad=None, padmode=PAD_PKCS5)
    secret_bytes = binascii.a2b_hex(bytes(value, encoding="utf8"))
    return str(des_obj.decrypt(secret_bytes, padmode=PAD_PKCS5), encoding="utf8")


# ------------------------生成密钥对------------------------
def create_rsa_pair(is_save=False):
    '''
    创建rsa公钥私钥对
    :param is_save: default:False
    :return: public_key, private_key
    '''
    f = RSA.generate(2048)
    private_key = f.exportKey("PEM")  # 生成私钥
    public_key = f.publickey().exportKey()  # 生成公钥
    if is_save:
        with open("crypto_private_key.pem", "wb") as f:
            f.write(private_key)
        with open("crypto_public_key.pem", "wb") as f:
            f.write(public_key)
    return public_key, private_key


def read_public_key(file_path="crypto_public_key.pem") -> bytes:
    with open(file_path, "rb") as x:
        b = x.read()
        return b


def read_private_key(file_path="crypto_private_key.pem") -> bytes:
    with open(file_path, "rb") as x:
        b = x.read()
        return b


# ------------------------加密------------------------
def rsa_encryption(text: str, public_key: bytes):
    # 字符串指定编码（转为bytes）
    text = text.replace('\r\n', '\n').encode('utf-8')
    key = RSA.importKey(public_key)
    # 构建公钥对象
    cipher_public = PKCS1_v1_5.new(key)
    # 加密（bytes）
    text_encrypted = cipher_public.encrypt(text)
    # base64编码，并转为字符串
    text_encrypted_base64 = base64.b64encode(text_encrypted).decode()
    return text_encrypted_base64


# ------------------------解密------------------------
def rsa_decryption(text_encrypted_base64: str, private_key: bytes):
    # 字符串指定编码（转为bytes）
    text_encrypted_base64 = text_encrypted_base64.encode('utf-8')
    # base64解码
    text_encrypted = base64.b64decode(text_encrypted_base64)
    # 构建私钥对象
    cipher_private = PKCS1_v1_5.new(RSA.importKey(private_key))
    # 解密（bytes）
    text_decrypted = cipher_private.decrypt(text_encrypted, Random.new().read)
    # 解码为字符串
    text_decrypted = text_decrypted.decode()
    return text_decrypted


if __name__ == '__main__':
    # 生成密钥对
    # create_rsa_pair(is_save=True)
    # public_key = read_public_key()
    # private_key = read_private_key()
    public_key, private_key = create_rsa_pair(is_save=False)

    # 加密
    text = 'Pickup@123456'
    public_key = """-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCsGAZ1trFEvH3TdHSFPbxCkdJUVLFsOUHW9ePVCAKkG0az4oqyKPJMME4avemlRlkBaNIpBPpqvPb0xBJgCu13ARm2YMNw7OdbHUiDjqciyU/WS5C9GiDGelHnSWM2m2sYdwU9zJO0I2nXViCccHTy29BWWkjXJTyCbiX2PyMymQIDAQAB
-----END PUBLIC KEY-----"""
    text_encrypted_base64 = rsa_encryption(text, public_key)
    print('密文：', text_encrypted_base64)

    # 解密
    # text_decrypted = rsa_decryption(text_encrypted_base64, private_key)
    # print('明文：', text_decrypted)






# coding=utf-8
import copy


class Dict(dict):
    def __init__(__self, *args, **kwargs):
        object.__setattr__(__self, '__parent', kwargs.pop('__parent', None))
        object.__setattr__(__self, '__key', kwargs.pop('__key', None))
        for arg in args:
            if not arg:
                continue
            elif isinstance(arg, dict):
                for key, val in arg.items():
                    __self[key] = __self._hook(val)
            elif isinstance(arg, tuple) and (not isinstance(arg[0], tuple)):
                __self[arg[0]] = __self._hook(arg[1])
            else:
                for key, val in iter(arg):
                    __self[key] = __self._hook(val)

        for key, val in kwargs.items():
            __self[key] = __self._hook(val)

    def __setattr__(self, name, value):
        if hasattr(self.__class__, name):
            raise AttributeError("'Dict' object attribute "
                                 "'{0}' is read-only".format(name))
        else:
            self[name] = value

    def __setitem__(self, name, value):
        super(Dict, self).__setitem__(name, value)
        try:
            p = object.__getattribute__(self, '__parent')
            key = object.__getattribute__(self, '__key')
        except AttributeError:
            p = None
            key = None
        if p is not None:
            p[key] = self
            object.__delattr__(self, '__parent')
            object.__delattr__(self, '__key')

    def __add__(self, other):
        if not self.keys():
            return other
        else:
            self_type = type(self).__name__
            other_type = type(other).__name__
            msg = "unsupported operand type(s) for +: '{}' and '{}'"
            raise TypeError(msg.format(self_type, other_type))

    @classmethod
    def _hook(cls, item):
        if isinstance(item, dict):
            return cls(item)
        elif isinstance(item, (list, tuple)):
            return type(item)(cls._hook(elem) for elem in item)
        return item

    def __getattr__(self, item):
        return self.__getitem__(item)

    def __missing__(self, name):
        return self.__class__(__parent=self, __key=name)

    def __delattr__(self, name):
        del self[name]

    def to_dict(self):
        base = {}
        for key, value in self.items():
            if isinstance(value, type(self)):
                base[key] = value.to_dict()
            elif isinstance(value, (list, tuple)):
                base[key] = type(value)(
                    item.to_dict() if isinstance(item, type(self)) else
                    item for item in value)
            else:
                base[key] = value
        return base

    def copy(self):
        return copy.copy(self)

    def deepcopy(self):
        return copy.deepcopy(self)

    def __deepcopy__(self, memo):
        other = self.__class__()
        memo[id(self)] = other
        for key, value in self.items():
            other[copy.deepcopy(key, memo)] = copy.deepcopy(value, memo)
        return other

    def update(self, *args, **kwargs):
        other = {}
        if args:
            if len(args) > 1:
                raise TypeError()
            other.update(args[0])
        other.update(kwargs)
        for k, v in other.items():
            if ((k not in self) or
                    (not isinstance(self[k], dict)) or
                    (not isinstance(v, dict))):
                self[k] = v
            else:
                self[k].update(v)

    def __getnewargs__(self):
        return tuple(self.items())

    def __getstate__(self):
        return self

    def __setstate__(self, state):
        self.update(state)

    def setdefault(self, key, default=None):
        if key in self:
            return self[key]
        else:
            self[key] = default
            return default


if __name__ == "__main__":
    a = {"success": True,
         "data": {"currentIndex": 1, "pageSize": 10, "totalNumber": 30, "totalPage": 3, "nextIndex": 2, "preIndex": 0,
                  "items2": [{'uuid2': "12345678"}]},
         "msg": "success", "code": 200, "traceId": None}
    b = Dict(a)

    print(b.data.items2[0].uuid2)

    a = {"success": True,
         "data": {"currentIndex": 1, "pageSize": 10, "totalNumber": 30, "totalPage": 3, "nextIndex": 2, "preIndex": 0,
                  "items": [{'uuid2': "12345678"}]},
         "msg": "success", "code": 200, "traceId": None}
    b = Dict(a)

    print(Dict(b.data["items"])[0].uuid2)



from setuptools import setup, find_packages

setup(
    name='huhangkai',  # 对外模块的名字
    version='1.1.7',  # 版本号
    description='接口自动化',  # 描述
    author='胡杭凯',  # 作者
    author_email='3173825608@qq.com',
    # package_dir={"": "commen"},
    packages=find_packages(),
    package_data={'by': ['常用命令.bat'],},
    python_requires=">=3.0",
    install_requires=[
        "faker",
        "openpyxl",
        "apscheduler",
        "rsa",
        "pyDes",
        "pycryptodome",
    ],
)


import re
from threading import Thread
from apscheduler.schedulers.blocking import BlockingScheduler


def shared_task(f):
    def wrapper(*args, **kwargs):
        thr = Thread(target=f, args=args, kwargs=kwargs)
        thr.start()

    return wrapper


class Scheduler:
    scheduler = None

    def __init__(self):
        if Scheduler.scheduler:
            try:
                Scheduler.scheduler.shutdown(False)
                Scheduler.scheduler = BlockingScheduler()
            except Exception as e:
                print(e)
            pass
        else:
            Scheduler.scheduler = BlockingScheduler()

    @staticmethod
    def get_cron(corn_str):
        cron_list = re.split(r'\s+', corn_str.strip())
        cron_list += ["*"] * 7
        second = cron_list[0]
        minute = cron_list[1]
        hour = cron_list[2]
        day = cron_list[3]
        month = cron_list[4]
        day_of_week = '*' if cron_list[5] in ('?', '？') else cron_list[5]
        year = cron_list[6]
        return second, minute, hour, day, month, day_of_week, year

    def add_jobs(self, fun, schedules=None, cron=None):
        if cron:
            second, minute, hour, day, month, day_of_week, year = self.get_cron(cron)
            if schedules:
                Scheduler.scheduler.add_job(fun, 'cron', year=year, day_of_week=day_of_week, month=month, day=day,
                                            hour=hour, minute=minute, second=second, args=(schedules,))
            else:
                Scheduler.scheduler.add_job(fun, 'cron', year=year, day_of_week=day_of_week, month=month, day=day,
                                            hour=hour, minute=minute, second=second)
            return True
        elif schedules:
            for schedule in schedules:
                if schedule.cron and schedule.cron.strip():
                    try:
                        second, minute, hour, day, month, day_of_week, year = self.get_cron(schedule.cron.strip())
                        Scheduler.scheduler.add_job(fun, 'cron', year=year, day_of_week=day_of_week, month=month,
                                                    day=day, hour=hour, minute=minute, second=second, args=(schedule,))
                    except Exception as e:
                        print("Error:" + str(e))
        return True

    @staticmethod
    def start():
        Scheduler.scheduler.start()